.jkit-notice-banner{position:relative;display:-webkit-box;display:-ms-flexbox;display:flex;margin:10px 20px 20px 0;padding:0;border:1px solid #c3c4c7;border-left-width:0}.jkit-notice-banner .jkit-notice-banner-logo{background:#ecf0f8;padding:20px 15px;border-left:4px;border-right:0;border-style:solid;-o-border-image:linear-gradient(to bottom,#3f3bf7,#5cd0da) 1 100%;border-image:-webkit-gradient(linear,left top,left bottom,from(#3f3bf7),to(#5cd0da)) 1 100%;border-image:linear-gradient(to bottom,#3f3bf7,#5cd0da) 1 100%}.jkit-notice-banner .jkit-notice-banner-logo img{width:24px;height:24px}.jkit-notice-banner .jkit-notice-banner-content{padding:20px 20px 25px 20px;width:100%}.jkit-notice-banner .jkit-notice-banner-content h2{margin-top:0;margin-bottom:10px}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;margin-top:20px}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action a{text-decoration:none}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-fix-action-button,.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-button{color:#fff;background:#1b67a5;border-radius:3px;cursor:pointer;padding:7px 15px;text-decoration:none}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-right{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;gap:5px}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-right a{display:-webkit-box;display:-ms-flexbox;display:flex;gap:7px;text-decoration:none;color:#1b67a5;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-right a:before{display:block;font-size:16px}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-left{display:-webkit-box;display:-ms-flexbox;display:flex;gap:20px;vertical-align:middle;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-left span{color:#ffb900}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-left .jkit-notice-close{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;gap:7px}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-left .jkit-notice-close svg *{fill:#2271b1}.jkit-notice-banner .jkit-notice-banner-content .jkit-notice-action .jkit-notice-action-left .jkit-notice-close:hover svg *{fill:#135e96}.jkit-notice-banner .jkit-banner-close{position:absolute;top:20px;right:20px}.jkit-notice-banner .jkit-banner-close svg{width:10px;height:10px;cursor:pointer}.jkit-notice-banner .jkit-banner-close:hover svg *{fill:#fd3d31}
/*# sourceMappingURL=notice-banner.css.map */
