#adminmenu .toplevel_page_metform-menu .wp-menu-image>img{-webkit-filter:grayscale(1);filter:grayscale(1);transition:opacity .3s,-webkit-filter .3s;transition:opacity .3s,filter .3s;transition:opacity .3s,filter .3s,-webkit-filter .3s}#adminmenu .toplevel_page_metform-menu.opensub .wp-menu-image>img,#adminmenu .toplevel_page_metform-menu.wp-menu-open .wp-menu-image>img,#adminmenu .toplevel_page_metform-menu:hover .wp-menu-image>img{-webkit-filter:grayscale(0);filter:grayscale(0)}@keyframes infinite-spinning{from{-webkit-transform:rotate(0);transform:rotate(0)}to{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}.metform-pdf-export-btn.loading{pointer-events:none;filter:grayscale(1)}.metform-pdf-export-btn.loading .pdf-spinner{position:relative;width:18px;height:18px;border-radius:18px;background:rgba(0,0,0,.5);-webkit-animation:infinite-spinning 1.08s linear infinite;animation:infinite-spinning 1.08s linear infinite;vertical-align:middle;margin-right:7px;display:inline-block;top:-1px;left:5px}.metform-pdf-export-btn.loading .pdf-spinner::after{content:"";position:absolute;top:2px;left:50%;width:4px;height:4px;border-radius:4px;margin-left:-2px;background:#fff}